import { Mock, mock, useFromMock } from '@alfa-client/test-utils';
import { AuthService } from './auth.service';
import { KeycloakService, KeycloakEvent, KeycloakEventType } from 'keycloak-angular';
import { of } from 'rxjs';
import { KeycloakProfile } from 'keycloak-js';
import { StateResource, createStateResource } from '@alfa-client/tech-shared';
import { UserProfileResource } from '@alfa-client/user-profile-shared';
import { createUserProfileResource } from '../../../../../libs/user-profile-shared/test/user-profile';

describe('AuthService', () => {
  let service: AuthService;
  let keycloakService: Mock<KeycloakService>;

  const event: KeycloakEvent = <any>{};

  beforeEach(() => {
    keycloakService = { ...mock(KeycloakService), keycloakEvents$: <any>of(event) };

    service = new AuthService(useFromMock(keycloakService));
  });

  describe('listenToKeycloakEvent', () => {
    it('should call handleEvent', () => {
      service.handleEvent = jest.fn();

      service.listenToKeycloakEvent();

      expect(service.handleEvent).toHaveBeenCalledWith(event);
    });
  });

  describe('handleEvent', () => {
    describe('on token expired event', () => {
      const tokenExpiredEvent: KeycloakEvent = { type: KeycloakEventType.OnTokenExpired };

      it('should updateToken', () => {
        service.handleEvent(tokenExpiredEvent);

        expect(keycloakService.updateToken).toHaveBeenCalledWith(10);
      });

      it('should call handleTokenExpireEvent', async () => {
        const updateTokenReturnValue: boolean = true;
        service.handleTokenExpiredEvent = jest.fn();
        keycloakService.updateToken.mockImplementation(() =>
          Promise.resolve(updateTokenReturnValue),
        );

        await service.handleEvent(tokenExpiredEvent);

        expect(service.handleTokenExpiredEvent).toHaveBeenCalledWith(updateTokenReturnValue);
      });
    });
  });

  describe('handleTokenExpiredEvent', () => {
    it('should getToken if refresh was success', () => {
      service.handleTokenExpiredEvent(true);

      expect(keycloakService.getToken).toHaveBeenCalled();
    });

    it('should store token', async () => {
      const token: string = 'tokenDummy';
      service.storeAccessToken = jest.fn();
      keycloakService.getToken.mockImplementation(() => Promise.resolve(token));

      await service.handleTokenExpiredEvent(true);

      expect(service.storeAccessToken).toHaveBeenCalledWith(token);
    });
  });

  describe('login', () => {
    it('should call isLoggedIn', () => {
      service.login();

      expect(keycloakService.isLoggedIn());
    });

    describe('after loggedIn', () => {
      const isLoggedIn: boolean = true;

      beforeEach(() => {
        keycloakService.isLoggedIn.mockImplementation(() => Promise.resolve(isLoggedIn));
        service.setCurrentUser = jest.fn();
      });
      it('should store token', async () => {
        const token: string = 'tokenDummy';
        keycloakService.getToken.mockImplementation(() => Promise.resolve(token));
        service.storeAccessToken = jest.fn();

        await service.login();

        expect(service.storeAccessToken).toHaveBeenCalledWith(token);
      });

      it('should set currentUser', async () => {
        await service.login();

        expect(service.setCurrentUser).toHaveBeenCalled();
      });
    });
    describe('if not logged in yet', () => {
      it('should call login', async () => {
        const isLoggedIn: boolean = false;
        keycloakService.isLoggedIn.mockImplementation(() => Promise.resolve(isLoggedIn));

        await service.login();

        expect(keycloakService.login).toHaveBeenCalledWith({ scope: 'offline_access' });
      });
    });
  });

  describe('store access token', () => {
    it('should put token in sessionStorage', () => {
      const token: string = 'tokenDummy';

      service.storeAccessToken(token);

      const accessTokenInSessionStorage: string = sessionStorage.getItem('access_token');

      expect(accessTokenInSessionStorage).toStrictEqual(token);
    });
  });
  describe('logout', () => {
    it('should call keycloakservice logout', () => {
      service.logout();

      expect(keycloakService.logout).toHaveBeenCalled();
    });
  });

  describe('set current user', () => {
    it('should load user profile', () => {
      service.setCurrentUser();

      expect(keycloakService.loadUserProfile).toHaveBeenCalled();
    });
    it('should update current user state resource', async () => {
      const profile: KeycloakProfile = { firstName: 'blubb', lastName: 'blabb' };
      keycloakService.loadUserProfile.mockImplementation(() => Promise.resolve(profile));

      await service.setCurrentUser();

      expect(service.currentUserStateResource()).toEqual(createStateResource(profile));
    });
  });

  describe('getCurrentUserInitials', () => {
    it('should return currentUserStateResource', () => {
      const userProfile: UserProfileResource = createUserProfileResource();
      const userStateResource: StateResource<UserProfileResource> =
        createStateResource(userProfile);
      service.currentUserStateResource.set(userStateResource);
      const currentUserInitials: string = service.getCurrentUserInitials();

      const initials: string =
        userProfile.firstName.substring(0, 1) + '' + userProfile.lastName.substring(0, 1);
      expect(currentUserInitials).toEqual(initials);
    });
  });
});
