/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { SnackBarE2EComponent } from '../../../components/ui/snackbar.e2e.component';
import { VorgangDetailHeaderE2EComponent } from '../../../components/vorgang/vorgang-detail-header.e2e.component';
import { VorgangListE2EComponent } from '../../../components/vorgang/vorgang-list.e2e.component';
import { EingangE2E, VorgangE2E } from '../../../model/vorgang';
import {
  MainPage,
  waitForSpinnerToDisappear,
  waitforSpinnerToAppear,
} from '../../../page-objects/main.po';
import { dropCollections, visitUrl } from '../../../support/cypress-helper';
import { contains, exist, notExist } from '../../../support/cypress.util';
import {
  ORGANISATIONSEINHEITEN_ID_FOR_KFINDER,
  ORGANISATIONSEINHEITEN_ID_FOR_KORDNER,
} from '../../../support/data.util';
import { MessagesE2E } from '../../../support/messages';
import { loginAsKfinder, loginAsKordner, loginAsZonk } from '../../../support/user-util';
import {
  buildVorgang,
  createVorgang,
  createVorgangUriById,
  initVorgaenge,
} from '../../../support/vorgang-util';

const defaultCommandTimeout: number = 30000;

describe('Vorgang-detailansicht filtered by organisationseinheit', () => {
  const mainPage: MainPage = new MainPage();
  const vorgangList: VorgangListE2EComponent = mainPage.getVorgangList();
  const detailPage: VorgangDetailHeaderE2EComponent = new VorgangDetailHeaderE2EComponent();
  const snackbar: SnackBarE2EComponent = mainPage.getSnackBar();

  const vorgangIdForKFinder: string = '601bc78eabc134051570aab8';
  const vorgangUrlVisibleToKFinder: string = createVorgangUriById(vorgangIdForKFinder);

  const eingangForKFinder: EingangE2E = {
    ...createVorgang().eingangs[0],
    zustaendigeStelle: { organisationseinheitenId: ORGANISATIONSEINHEITEN_ID_FOR_KFINDER },
  };
  const vorgangForKFinder: VorgangE2E = {
    ...buildVorgang(vorgangIdForKFinder, 'VorgangVisibleToKFinder'),
    eingangs: [eingangForKFinder],
  };

  const vorgangIdForKOrdner: string = '60250b9d383c182943f6ba79';
  const vorgangUrlVisibleToKOrdner: string = createVorgangUriById(vorgangIdForKOrdner);

  const eingangForKOrdner: EingangE2E = {
    ...createVorgang().eingangs[0],
    zustaendigeStelle: { organisationseinheitenId: ORGANISATIONSEINHEITEN_ID_FOR_KORDNER },
  };
  const vorgangForKOrdner: VorgangE2E = {
    ...buildVorgang(vorgangIdForKOrdner, 'VorgangVisibleToKOrdner'),
    eingangs: [eingangForKOrdner],
  };

  before(() => {
    initVorgaenge([vorgangForKFinder, vorgangForKOrdner]);
  });

  after(() => {
    dropCollections();
  });

  describe('on user kfinder', () => {
    const authorizedUrl: string = vorgangUrlVisibleToKFinder;
    const forbiddenUrl: string = vorgangUrlVisibleToKOrdner;

    before(() => {
      loginAsKfinder();
    });

    it('should show vorgangList after login', () => {
      waitforSpinnerToAppear();
      waitForSpinnerToDisappear();
      exist(vorgangList.getRoot());
    });

    it('should open detailpage', { defaultCommandTimeout }, () => {
      visitUrl(authorizedUrl);

      waitforSpinnerToAppear();
      waitForSpinnerToDisappear();

      exist(detailPage.getRoot());
    });

    it('should not open detailpage', { defaultCommandTimeout }, () => {
      visitUrl(forbiddenUrl);

      waitforSpinnerToAppear();
      waitForSpinnerToDisappear();

      notExist(detailPage.getRoot());
    });

    it('should show snackbar', () => {
      exist(snackbar.getCloseButton());
      contains(snackbar.getMessage(), MessagesE2E.HTTP_STATUS_FORBIDDEN);
    });
  });

  describe('on user kordner', () => {
    const authorizedUrl: string = vorgangUrlVisibleToKOrdner;
    const forbiddenUrl: string = vorgangUrlVisibleToKFinder;

    before(() => {
      loginAsKordner();
    });

    it('should show vorgangList after login', () => {
      waitforSpinnerToAppear();
      waitForSpinnerToDisappear();
      exist(vorgangList.getRoot());
    });

    it('should open detailpage', { defaultCommandTimeout }, () => {
      visitUrl(authorizedUrl);

      waitforSpinnerToAppear();
      waitForSpinnerToDisappear();

      exist(detailPage.getRoot());
    });

    it('should not open detailpage', { defaultCommandTimeout }, () => {
      visitUrl(forbiddenUrl);

      waitforSpinnerToAppear();
      waitForSpinnerToDisappear();

      notExist(detailPage.getRoot());
    });

    it('should show snackbar', () => {
      exist(snackbar.getCloseButton());
      contains(snackbar.getMessage(), MessagesE2E.HTTP_STATUS_FORBIDDEN);
    });
  });

  describe('on user zonk', () => {
    const authorizedUrl: string = vorgangUrlVisibleToKOrdner;
    const forbiddenUrl: string = vorgangUrlVisibleToKFinder;

    before(() => {
      loginAsZonk();

      waitforSpinnerToAppear();
      waitForSpinnerToDisappear();
      exist(vorgangList.getRoot());
    });

    describe('for vorgang with assigned organisationseinheitenId', () => {
      it('should not open detailpage (for kordner)', () => {
        visitUrl(authorizedUrl);

        waitforSpinnerToAppear();
        waitForSpinnerToDisappear();

        notExist(detailPage.getRoot());
      });

      it('should show snackbar', () => {
        exist(snackbar.getCloseButton());
        contains(snackbar.getMessage(), MessagesE2E.HTTP_STATUS_FORBIDDEN);
      });

      it('should close snackbar on close button', () => {
        //FIXME Workaround, somewhere and somehow here are appearing multiple instances of snackbar(or the button)
        snackbar.getCloseButton().click({ multiple: true });

        notExist(snackbar.getCloseButton());
        notExist(snackbar.getMessage());
      });

      it('should not open detailpage (for kfinder)', () => {
        visitUrl(forbiddenUrl);

        waitforSpinnerToAppear();
        waitForSpinnerToDisappear();

        notExist(detailPage.getRoot());
      });

      it('should show snackbar', () => {
        exist(snackbar.getCloseButton());
        contains(snackbar.getMessage(), MessagesE2E.HTTP_STATUS_FORBIDDEN);
      });
    });
  });
});
