/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { NavigationE2EComponent } from 'apps/alfa-e2e/src/components/navigation/navigation.e2e.component';
import { MainPage, waitForSpinnerToDisappear } from 'apps/alfa-e2e/src/page-objects/main.po';
import { Interception } from 'cypress/types/net-stubbing';
import { HttpMethodE2E } from '../../../model/util';
import { VorgangE2E, VorgangStatusE2E } from '../../../model/vorgang';
import {
  dropCollections,
  intercept,
  scrollToWindowBottom,
  waitOfInterceptor,
} from '../../../support/cypress-helper';
import { generateIds } from '../../../support/tech.util';
import {
  getUserManagerUserSabine,
  getUserSabineInternalId,
  initUsermanagerUsers,
  loginAsSabine,
} from '../../../support/user-util';
import { buildVorgang, initVorgaenge } from '../../../support/vorgang-util';

describe('VorgangList Pages', () => {
  const mainPage: MainPage = new MainPage();

  const navigation: NavigationE2EComponent = mainPage.getNavigation();

  const PAGE_SIZE: number = 100;
  const assignedTo: string = getUserSabineInternalId();

  before(() => {
    const vorgaenge: VorgangE2E[] = createVorgaengeWithStatus(VorgangStatusE2E.NEU).concat(
      createVorgaengeWithStatusAndAssignedTo(VorgangStatusE2E.NEU),
      createVorgaengeWithStatus(VorgangStatusE2E.ANGENOMMEN),
      createVorgaengeWithStatusAndAssignedTo(VorgangStatusE2E.ANGENOMMEN),
    );
    initVorgaenge(vorgaenge);
    initUsermanagerUsers([getUserManagerUserSabine()]);

    loginAsSabine();
  });

  after(() => {
    dropCollections();
  });

  describe('VorgangList', () => {
    describe('with Alle Filter', () => {
      describe('1 page (NEU)', () => {
        const loadPageInterceptor: string = 'loadPage';

        beforeEach(() => {
          intercept(HttpMethodE2E.GET, buildVorgangPageLink(1)).as(loadPageInterceptor);
        });

        it('should show 1 page with 100 vorgaenge NEU', () => {
          waitOfInterceptor(loadPageInterceptor).then((interception: Interception) => {
            assert(interception.response.body._embedded.vorgangHeaderList.length === PAGE_SIZE);
            assert(
              interception.response.body._embedded.vorgangHeaderList[0].name === 'VorgangNEU_1',
            );
            assert(
              interception.response.body._embedded.vorgangHeaderList[99].name ===
                'VorgangNEU_' + PAGE_SIZE,
            );
          });
        });
      });

      describe('2 page (NEU, ASSIGNED)', () => {
        const loadNextPageInterceptor: string = 'loadNextPage';

        beforeEach(() => {
          intercept(HttpMethodE2E.GET, buildVorgangPageLink(2)).as(loadNextPageInterceptor);
        });

        it('should show 2 page with 100 vorgaenge NEU ASSIGNED on scrolling to bottom', () => {
          scrollToWindowBottom();

          waitOfInterceptor(loadNextPageInterceptor).then((interception: Interception) => {
            assert(interception.response.body._embedded.vorgangHeaderList.length === PAGE_SIZE);
            assert(
              interception.response.body._embedded.vorgangHeaderList[0].name ===
                'VorgangNEUAssigned_1',
            );
            assert(
              interception.response.body._embedded.vorgangHeaderList[99].name ===
                'VorgangNEUAssigned_' + PAGE_SIZE,
            );
          });
          waitForSpinnerToDisappear();
        });
      });

      describe('3 page (ANGENOMMEN)', () => {
        const loadNextPageInterceptor: string = 'loadNextPage';

        beforeEach(() => {
          intercept(HttpMethodE2E.GET, buildVorgangPageLink(3)).as(loadNextPageInterceptor);
        });

        it('should show 2 page with 100 vorgaenge ANGENOMMEN on scrolling to bottom', () => {
          scrollToWindowBottom();

          waitOfInterceptor(loadNextPageInterceptor).then((interception: Interception) => {
            assert(interception.response.body._embedded.vorgangHeaderList.length === PAGE_SIZE);
            assert(
              interception.response.body._embedded.vorgangHeaderList[0].name ===
                'VorgangANGENOMMEN_1',
            );
            assert(
              interception.response.body._embedded.vorgangHeaderList[99].name ===
                'VorgangANGENOMMEN_' + PAGE_SIZE,
            );
          });
          waitForSpinnerToDisappear();
        });
      });

      describe('4 page (ANGENOMMEN, ASSIGNED)', () => {
        const loadNextPageInterceptor: string = 'loadNextPage';

        beforeEach(() => {
          intercept(HttpMethodE2E.GET, buildVorgangPageLink(4)).as(loadNextPageInterceptor);
        });

        it('should show 3 page with 100 vorgaenge ANGENOMMEN ASSIGNED on scrolling to bottom', () => {
          scrollToWindowBottom();

          waitOfInterceptor(loadNextPageInterceptor).then((interception: Interception) => {
            scrollToWindowBottom();
            assert(interception.response.body._embedded.vorgangHeaderList.length === PAGE_SIZE);
            assert(
              interception.response.body._embedded.vorgangHeaderList[0].name ===
                'VorgangANGENOMMENAssigned_1',
            );
            assert(
              interception.response.body._embedded.vorgangHeaderList[99].name ===
                'VorgangANGENOMMENAssigned_' + PAGE_SIZE,
            );
          });
        });
      });

      function buildVorgangPageLink(page: number): string {
        return `*/vorgangs?page=${page - 1}&limit=${PAGE_SIZE}`;
      }
    });

    describe('with MeineVorgaenge Filter', () => {
      describe('1 page (NEU ASSIGNED)', () => {
        const loadPageInterceptor: string = 'loadPage';

        beforeEach(() => {
          intercept(HttpMethodE2E.GET, buildVorgangPageLink(1)).as(loadPageInterceptor);
        });

        it('should show 1 page with 100 vorgaenge NEU ASSIGNED', () => {
          navigation.getMeineVorgaengeFilter().click();

          waitOfInterceptor(loadPageInterceptor).then((interception: Interception) => {
            assert(interception.response.body._embedded.vorgangHeaderList.length === PAGE_SIZE);
            assert(
              interception.response.body._embedded.vorgangHeaderList[0].name ===
                'VorgangNEUAssigned_1',
            );
            assert(
              interception.response.body._embedded.vorgangHeaderList[99].name ===
                'VorgangNEUAssigned_' + PAGE_SIZE,
            );
          });
        });
      });

      describe('2 page (ANGENOMMEN, ASSIGNED)', () => {
        const loadNextPageInterceptor: string = 'loadNextPage';

        beforeEach(() => {
          intercept(HttpMethodE2E.GET, buildVorgangPageLink(2)).as(loadNextPageInterceptor);
        });

        it('should show 2 page with 100 vorgaenge ANGENOMMEN ASSIGNED on scrolling to bottom', () => {
          scrollToWindowBottom();

          waitOfInterceptor(loadNextPageInterceptor).then((interception: Interception) => {
            assert(interception.response.body._embedded.vorgangHeaderList.length === PAGE_SIZE);
            assert(
              interception.response.body._embedded.vorgangHeaderList[0].name ===
                'VorgangANGENOMMENAssigned_1',
            );
            assert(
              interception.response.body._embedded.vorgangHeaderList[99].name ===
                'VorgangANGENOMMENAssigned_' + PAGE_SIZE,
            );
          });
          waitForSpinnerToDisappear();
        });
      });

      function buildVorgangPageLink(page: number): string {
        return `*/vorgangs?page=${page - 1}&limit=${PAGE_SIZE}&assignedTo=${assignedTo}`;
      }
    });
  });

  function createVorgaengeWithStatus(status: VorgangStatusE2E): VorgangE2E[] {
    let count = 1;
    return generateIds(100).map((randomId) => {
      const vorgang: VorgangE2E = {
        ...buildVorgang(randomId, 'Vorgang' + status + '_' + count),
        status,
      };
      count++;
      return vorgang;
    });
  }

  function createVorgaengeWithStatusAndAssignedTo(status: VorgangStatusE2E): VorgangE2E[] {
    let count = 1;
    return generateIds(100).map((randomId) => {
      const vorgang: VorgangE2E = {
        ...buildVorgang(randomId, 'Vorgang' + status + 'Assigned_' + count),
        status,
        assignedTo,
      };
      count++;
      return vorgang;
    });
  }
});
