/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */

import { EMPTY_ARRAY } from '@alfa-client/tech-shared';
import { Resource } from '@ngxp/rest';
import {
  createDummyListResource,
  createDummyResource,
  toResource,
} from 'libs/tech-shared/test/resource';
import { DummyListLinkRel } from '../../../test/dummy';
import {
  createEmptyStateResource,
  createStateResource,
  getEmbeddedResources,
  isLoaded,
  isLoadingRequired,
  isValidStateResouce,
} from './resource.util';

describe('resource util', () => {
  describe('isLoadingRequired', () => {
    it('should return true if not loaded', () => {
      const isRequired: boolean = isLoadingRequired(createEmptyStateResource());

      expect(isRequired).toBe(true);
    });

    it('should retrun false if already loading', () => {
      const isRequired: boolean = isLoadingRequired(createEmptyStateResource(true));

      expect(isRequired).toBe(false);
    });

    it('should return false on reload if already loading', () => {
      const isRequired: boolean = isLoadingRequired({
        ...createEmptyStateResource(true),
        reload: true,
      });

      expect(isRequired).toBe(false);
    });

    it('should return true if reload requested', () => {
      const isRequired: boolean = isLoadingRequired({
        ...createStateResource(toResource({})),
        reload: true,
      });

      expect(isRequired).toBe(true);
    });
  });

  describe('isLoaded', () => {
    it('should return false if not loaded', () => {
      const loaded: boolean = isLoaded(createEmptyStateResource());

      expect(loaded).toBe(false);
    });

    it('should retrun false if loading', () => {
      const loaded: boolean = isLoaded(createEmptyStateResource(true));

      expect(loaded).toBe(false);
    });

    it('should return false on reload if already loading', () => {
      const loaded: boolean = isLoaded({ ...createEmptyStateResource(true), reload: true });

      expect(loaded).toBe(false);
    });

    it('should return false if reload required', () => {
      const loaded: boolean = isLoaded({ ...createStateResource(toResource({})), reload: true });

      expect(loaded).toBe(false);
    });

    it('should return true if resource is present and loaded', () => {
      const loaded: boolean = isLoaded(createStateResource(toResource({})));

      expect(loaded).toBe(true);
    });
  });

  describe('isValidStateResouce', () => {
    it('should return false if not loaded', () => {
      const isValid: boolean = isValidStateResouce(createEmptyStateResource());

      expect(isValid).toBe(false);
    });

    it('should retrun false if loaded but is null', () => {
      const isValid: boolean = isValidStateResouce(createStateResource(null));

      expect(isValid).toBe(false);
    });

    it('should return false on loading', () => {
      const isValid: boolean = isValidStateResouce(createEmptyStateResource(true));

      expect(isValid).toBe(false);
    });

    it('should retrun true if already loaded', () => {
      const isValid: boolean = isValidStateResouce({
        ...createStateResource(createDummyResource()),
      });

      expect(isValid).toBe(true);
    });
  });

  describe('getEmbeddedResources', () => {
    it('should return empty array if state resource null', () => {
      const embedded = getEmbeddedResources(null, null);

      expect(embedded).toEqual(EMPTY_ARRAY);
    });

    it('should return empty array if resource null', () => {
      const embedded = getEmbeddedResources(createEmptyStateResource(), null);

      expect(embedded).toEqual(EMPTY_ARRAY);
    });

    it('should return null if embedded relation does not exist', () => {
      const stateResource = createStateResource<Resource>(createDummyResource());

      const embedded = getEmbeddedResources<string[]>(stateResource, 'users');

      expect(embedded).toBeNull();
    });

    it('should return embedded resource', () => {
      const dummyListResource = createDummyListResource();
      const stateResource = createStateResource<Resource>(dummyListResource);

      const embedded = getEmbeddedResources<Resource>(stateResource, DummyListLinkRel.LIST);

      expect(embedded).toEqual(dummyListResource._embedded[DummyListLinkRel.LIST]);
    });
  });
});
