import {
  getElementFromDomRoot,
  getElementFromFixture,
  getMockComponent,
} from '@alfa-client/test-utils';
import { MenuItemComponent, OzgcloudIconComponent, UiModule } from '@alfa-client/ui';
import { VorgangWithEingangLinkRel, VorgangWithEingangResource } from '@alfa-client/vorgang-shared';
import { ChangeDetectorRef, SimpleChanges } from '@angular/core';
import { ComponentFixture, TestBed } from '@angular/core/testing';
import { NoopAnimationsModule } from '@angular/platform-browser/animations';
import { getDataTestIdOf } from 'libs/tech-shared/test/data-test';
import { createVorgangWithEingangResource } from 'libs/vorgang-shared/test/vorgang';
import { MockComponent } from 'ng-mocks';
import { VorgangDetailMoreMenuComponent } from './vorgang-detail-more-menu.component';
import { VorgangExportContainerComponent } from './vorgang-export-container/vorgang-export-container.component';
import { ProcessVorgangContainerComponent } from './vorgang-process-vorgang-container/vorgang-process-vorgang-container.component';

describe('VorgangDetailMoreMenuComponent', () => {
  let component: VorgangDetailMoreMenuComponent;
  let fixture: ComponentFixture<VorgangDetailMoreMenuComponent>;

  const moreMenuButton: string = getDataTestIdOf('more-menu-button');
  const exportMenuItem: string = getDataTestIdOf('vorgang-exportieren-button');
  const processVorgangMenuItem: string = getDataTestIdOf('vorgang-process-vorgang-button');

  beforeEach(async () => {
    await TestBed.configureTestingModule({
      declarations: [
        VorgangDetailMoreMenuComponent,
        MockComponent(MenuItemComponent),
        MockComponent(OzgcloudIconComponent),
        MockComponent(VorgangExportContainerComponent),
        MockComponent(ProcessVorgangContainerComponent),
      ],
      imports: [UiModule, NoopAnimationsModule],
    }).compileComponents();

    fixture = TestBed.createComponent(VorgangDetailMoreMenuComponent);
    component = fixture.componentInstance;
    fixture.detectChanges();
  });

  it('should create', () => {
    expect(component).toBeTruthy();
  });

  describe('ngOnChanges', () => {
    it('should call vorgangWithEingangChanged if vorgangWithEingang changed', () => {
      const simpleChanges: SimpleChanges = <any>{ vorgangWithEingang: {} };
      const spy = jest.spyOn(component, 'vorgangWithEingangChanged');

      component.ngOnChanges(simpleChanges);

      expect(spy).toHaveBeenCalled();
    });

    it('should not call vorgangWithEingangChanged if vorgangWithEingang did not change', () => {
      const simpleChanges: SimpleChanges = <any>{ dummy: {} };
      const spy = jest.spyOn(component, 'vorgangWithEingangChanged');

      component.ngOnChanges(simpleChanges);

      expect(spy).not.toHaveBeenCalled();
    });
  });

  describe('vorgangWithEingangChanged', () => {
    it('should call detectChanges', () => {
      const changeDetectorRef = fixture.debugElement.injector.get(ChangeDetectorRef);
      const detectChangesSpy = jest.spyOn(changeDetectorRef.constructor.prototype, 'detectChanges');

      component.vorgangWithEingangChanged();

      expect(detectChangesSpy).toHaveBeenCalled();
    });

    it('should call isButtonDisabled', () => {
      const spy = jest.spyOn(component, 'isButtonEnabled');

      component.vorgangWithEingangChanged();

      expect(spy).toHaveBeenCalled();
    });
  });

  describe('isButtonEnabled', () => {
    it('should return true if at least 1 menu item exists', () => {
      component.menuItems.reset([new MenuItemComponent()]);

      const res: boolean = component.isButtonEnabled();

      expect(res).toBeTruthy();
    });

    it('should return false if no menu item exists', () => {
      component.menuItems.reset([]);

      const res: boolean = component.isButtonEnabled();

      expect(res).toBeFalsy();
    });
  });

  describe('closeMenu', () => {
    //TODO Auf das event vom Button ausweiten
    it.skip('should closeMenu by trigger', () => {
      jest.spyOn(component.trigger, 'closeMenu');

      component.closeMenu();

      expect(component.trigger.closeMenu).toHaveBeenCalled();
    });
  });

  describe('export menu item', () => {
    const vorgangWithEingang: VorgangWithEingangResource = createVorgangWithEingangResource([
      VorgangWithEingangLinkRel.EXPORT,
    ]);

    beforeEach(() => {
      component.buttonEnabled = true;
      fixture.detectChanges();
    });

    it('should be visible if link exists', () => {
      component.vorgangWithEingang = vorgangWithEingang;
      fixture.detectChanges();
      getElementFromFixture(fixture, moreMenuButton).click();

      const element = getElementFromDomRoot(fixture, exportMenuItem);

      expect(element).toBeInTheDocument();
    });

    it('should NOT be visible if link is missing', () => {
      component.vorgangWithEingang = createVorgangWithEingangResource();
      fixture.detectChanges();
      getElementFromFixture(fixture, moreMenuButton).click();

      const element = getElementFromDomRoot(fixture, exportMenuItem);

      expect(element).not.toBeInTheDocument();
    });

    describe('input property', () => {
      beforeEach(() => {
        component.vorgangWithEingang = vorgangWithEingang;
        fixture.detectChanges();
      });

      it('should contains headline', () => {
        getElementFromFixture(fixture, moreMenuButton).click();

        const menuItem: MenuItemComponent = getMockComponent(fixture, MenuItemComponent);

        expect(menuItem.headline).toBe('Vorgang exportieren');
      });

      it('should contains text', () => {
        getElementFromFixture(fixture, moreMenuButton).click();

        const menuItem: MenuItemComponent = getMockComponent(fixture, MenuItemComponent);

        expect(menuItem.text).toBe(
          'Alle Informationen und Anhänge des Vorgangs zur Archivierung im DMS.',
        );
      });

      it('should contains icon', () => {
        getElementFromFixture(fixture, moreMenuButton).click();

        const menuItem: MenuItemComponent = getMockComponent(fixture, MenuItemComponent);

        expect(menuItem.icon).toBe('xdomea_file');
      });

      it('should contains iconSizeBig', () => {
        getElementFromFixture(fixture, moreMenuButton).click();

        const menuItem: MenuItemComponent = getMockComponent(fixture, MenuItemComponent);

        expect(menuItem.iconSizeBig).toBeTruthy();
      });

      describe('for button', () => {
        it.skip('should contains vorgangWithEingang', () => {
          getElementFromFixture(fixture, moreMenuButton).click();

          const button: ProcessVorgangContainerComponent = getMockComponent(
            fixture,
            ProcessVorgangContainerComponent,
          );

          expect(button.vorgangWithEingang).toBe(vorgangWithEingang);
        });
      });
    });
  });

  describe('processVorgang menu item', () => {
    const vorgangWithEingang: VorgangWithEingangResource = createVorgangWithEingangResource([
      VorgangWithEingangLinkRel.PROCESS_VORGANG,
    ]);

    beforeEach(() => {
      component.buttonEnabled = true;
      fixture.detectChanges();
    });

    it('should be visible if link exists', () => {
      component.vorgangWithEingang = vorgangWithEingang;
      fixture.detectChanges();
      getElementFromFixture(fixture, moreMenuButton).click();

      const element = getElementFromDomRoot(fixture, processVorgangMenuItem);

      expect(element).toBeInTheDocument();
    });

    it('should NOT be visible if link is missing', () => {
      component.vorgangWithEingang = createVorgangWithEingangResource();
      fixture.detectChanges();
      getElementFromFixture(fixture, moreMenuButton).click();

      const element = getElementFromDomRoot(fixture, processVorgangMenuItem);

      expect(element).not.toBeInTheDocument();
    });

    describe('input property', () => {
      beforeEach(() => {
        component.vorgangWithEingang = vorgangWithEingang;
        fixture.detectChanges();
      });

      it('should contains headline', () => {
        getElementFromFixture(fixture, moreMenuButton).click();

        const menuItem: MenuItemComponent = getMockComponent(fixture, MenuItemComponent);

        expect(menuItem.headline).toBe('Vorgang automatisiert vorprüfen');
      });

      it('should contains text', () => {
        getElementFromFixture(fixture, moreMenuButton).click();

        const menuItem: MenuItemComponent = getMockComponent(fixture, MenuItemComponent);

        expect(menuItem.text).toBe(
          'Eine Vorprüfung wird durchgeführt. Das Ergebnis wird als Kommentar hinzugefügt.',
        );
      });

      it('should contains icon', () => {
        getElementFromFixture(fixture, moreMenuButton).click();

        const menuItem: MenuItemComponent = getMockComponent(fixture, MenuItemComponent);

        expect(menuItem.icon).toBe('vorgang_vorpruefen');
      });

      it('should contains iconSizeBig', () => {
        getElementFromFixture(fixture, moreMenuButton).click();

        const menuItem: MenuItemComponent = getMockComponent(fixture, MenuItemComponent);

        expect(menuItem.iconSizeBig).toBeTruthy();
      });

      describe('for button', () => {
        it.skip('should contains vorgangWithEingang', () => {
          getElementFromFixture(fixture, moreMenuButton).click();

          const button: VorgangExportContainerComponent = getMockComponent(
            fixture,
            VorgangExportContainerComponent,
          );

          expect(button.vorgangWithEingang).toBe(vorgangWithEingang);
        });
      });
    });
  });
});
