/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { ApiRootResource } from '@alfa-client/api-root-shared';
import { BinaryFileListResource } from '@alfa-client/binary-file-shared';
import {
  CommandListResource,
  CommandOrder,
  CommandResource,
  CreateCommand,
} from '@alfa-client/command-shared';
import { RouteData } from '@alfa-client/navigation-shared';
import {
  ApiError,
  ApiErrorAction,
  EMPTY_ARRAY,
  EMPTY_STRING,
  StateResource,
  createEmptyStateResource,
  createStateResource,
} from '@alfa-client/tech-shared';
import { HttpErrorResponse } from '@angular/common/http';
import { UrlSegment } from '@angular/router';
import { Action } from '@ngrx/store';
import { TypedAction } from '@ngrx/store/src/models';
import { Resource, ResourceUri } from '@ngxp/rest';
import { createApiRootResource } from 'libs/api-root-shared/test/api-root';
import { createBinaryFileListResource } from 'libs/binary-file-shared/test/binary-file';
import {
  createCommand,
  createCommandListResource,
  createCommandResource,
} from 'libs/command-shared/test/command';
import { createRouteData } from 'libs/navigation-shared/test/navigation-test-factory';
import { createDummyResource } from 'libs/tech-shared/test/resource';
import {
  createVorgangListResource,
  createVorgangListResourceWithResource,
  createVorgangResource,
  createVorgangResources,
  createVorgangStatistic,
  createVorgangWithEingangResource,
} from 'libs/vorgang-shared/test/vorgang';
import { createApiError, createHttpErrorResponse } from '../../../../tech-shared/test/error';
import {
  MEINE_ROUTE_PARAM,
  ROUTE_PARAM_BY_VORGANG_FILTER,
  ROUTE_PARAM_BY_VORGANG_VIEW,
  VORGANG_WITH_EINGANG_ROUTE_PARAM,
} from '../vorgang-navigation.util';
import { VorgangListLinkRel } from '../vorgang.linkrel';
import {
  StatusCommandMap,
  VorgangFilter,
  VorgangListResource,
  VorgangResource,
  VorgangStatistic,
  VorgangView,
  VorgangWithEingangResource,
} from '../vorgang.model';
import { VorgangListAction } from './vorgang.actions';
import { VorgangState, initialState, reducer } from './vorgang.reducer';

import faker from '@faker-js/faker';

import * as Storage from '@alfa-client/app-shared';
import * as CommandActions from '@alfa-client/command-shared';
import * as NavigationActions from '@alfa-client/navigation-shared';
import * as VorgangActions from './vorgang.actions';
import * as Reducer from './vorgang.reducer';

jest.mock('@alfa-client/app-shared', () => mockAsEsModule('@alfa-client/app-shared'));

function mockAsEsModule(module: string) {
  return {
    __esModule: true,
    ...jest.requireActual(module),
  };
}

describe('Vorgang Reducer', () => {
  describe('unknown action', () => {
    it('should return current state', () => {
      const action = {} as Action;

      const result = reducer(initialState, action);

      expect(result).toBe(initialState);
    });
  });

  describe('loadVorgangList', () => {
    describe('on "loadVorgangList" action', () => {
      it('should set loading to true', () => {
        const action = VorgangActions.loadVorgangList({
          apiRoot: createApiRootResource(),
          linkRel: 'dummyLinkRel',
        });

        const state: VorgangState = reducer(initialState, action);

        expect(state.vorgangList.loading).toBeTruthy();
      });
    });

    describe('on "loadVorgangListSuccess" action', () => {
      const vorgaenge: VorgangResource[] = createVorgangResources();
      const statistic: VorgangStatistic = createVorgangStatistic();
      const vorgangList: VorgangListResource = {
        ...createVorgangListResourceWithResource(vorgaenge, [VorgangListLinkRel.NEXT]),
        statistic,
      };
      const action: VorgangListAction & TypedAction<string> = VorgangActions.loadVorgangListSuccess(
        { vorgangList },
      );

      it('should set loaded resource', () => {
        const state: VorgangState = reducer(initialState, action);

        expect(state.vorgangList).toEqual(createStateResource(vorgangList));
      });

      it('should set vorgaenge', () => {
        const state: VorgangState = reducer(initialState, action);

        expect(state.vorgaenge.length).toBe(10);
        expect(state.vorgaenge).toStrictEqual(vorgaenge);
      });

      it('should clear searchPreviewList', () => {
        const state: VorgangState = reducer(
          { ...initialState, searchPreviewList: createStateResource(createVorgangListResource()) },
          action,
        );

        expect(state.searchPreviewList).toEqual(createEmptyStateResource());
      });

      it('should set vorgang statistic', () => {
        const state: VorgangState = reducer(initialState, action);

        expect(state.vorgangStatistic).toEqual(createStateResource(statistic));
      });
    });

    describe('on "loadVorgangListFailure" action', () => {
      const apiError: ApiError = createApiError();
      const action: ApiErrorAction & TypedAction<string> = VorgangActions.loadVorgangListFailure({
        apiError,
      });

      it('should set apiError', () => {
        const state: VorgangState = reducer(initialState, action);

        expect(state.vorgangList.error).toStrictEqual(apiError);
      });

      it('should clear searchPreviewList', () => {
        const state: VorgangState = reducer(initialState, action);

        expect(state.searchPreviewList).toEqual(createEmptyStateResource());
      });
    });
  });

  describe('loadNextPage', () => {
    describe('on "loadNextPage" action', () => {
      it('should set vorgangList loading to true', () => {
        const action = VorgangActions.loadNextPage();

        const state: VorgangState = reducer(initialState, action);

        expect(state.vorgangList.loading).toBeTruthy();
      });
    });

    describe('on "loadNextPageSuccess" action', () => {
      const statistic: VorgangStatistic = createVorgangStatistic();
      const vorgangList: VorgangListResource = { ...createVorgangListResource(), statistic };
      const action: VorgangListAction & TypedAction<string> = VorgangActions.loadNextPageSuccess({
        vorgangList,
      });

      it('should set vorgangList', () => {
        const state: VorgangState = reducer(initialState, action);

        expect(state.vorgangList).toStrictEqual(createStateResource(vorgangList));
      });

      it('should add vorgaenge', () => {
        const state: VorgangState = reducer(
          { ...initialState, vorgaenge: [createVorgangResource()] },
          action,
        );

        expect(state.vorgaenge.length).toBe(11);
      });

      it('should set vorgang statistic', () => {
        const state: VorgangState = reducer(initialState, action);

        expect(state.vorgangStatistic).toEqual(createStateResource(statistic));
      });
    });
  });

  describe('searchVorgaengeBy', () => {
    describe('on "searchVorgaengeBy" action', () => {
      const apiRoot: ApiRootResource = createApiRootResource();
      const searchString: string = 'search like me';
      const linkRel: string = 'LinkRelationName';

      const action = VorgangActions.searchVorgaengeBy({ apiRoot, searchString, linkRel });

      it('should vorgangList loading', () => {
        const state: VorgangState = reducer(initialState, action);

        expect(state.vorgangList.loading).toBeTruthy();
      });

      it('should clear searchPreviewList', () => {
        const state: VorgangState = reducer(initialState, action);

        expect(state.searchPreviewList).toStrictEqual(createEmptyStateResource());
      });
    });

    describe('on "searchVorgaengeBySuccess" action', () => {
      const vorgaenge: VorgangResource[] = createVorgangResources();
      const statistic: VorgangStatistic = createVorgangStatistic();
      const vorgangList: VorgangListResource = {
        ...createVorgangListResourceWithResource(vorgaenge),
        statistic,
      };
      const action: VorgangListAction & TypedAction<string> =
        VorgangActions.searchVorgaengeBySuccess({ vorgangList });

      it('should set vorgangList', () => {
        const state: VorgangState = reducer(initialState, action);

        expect(state.vorgangList).toStrictEqual(createStateResource(vorgangList));
      });

      it('should set vorgaenge', () => {
        const state: VorgangState = reducer(initialState, action);

        expect(state.vorgaenge).toBe(vorgaenge);
      });

      it('should clear searchPreviewList', () => {
        const state: VorgangState = reducer(initialState, action);

        expect(state.searchPreviewList).toStrictEqual(createEmptyStateResource());
      });

      it('should set vorgang statistic', () => {
        const state: VorgangState = reducer(initialState, action);

        expect(state.vorgangStatistic).toEqual(createStateResource(statistic));
      });
    });

    describe('on "searchVorgaengeByFailure" action', () => {
      const apiError: ApiError = createApiError();
      const httpErrorResponse: HttpErrorResponse = createHttpErrorResponse(apiError);
      const action = VorgangActions.searchVorgaengeByFailure({ httpErrorResponse });

      it('should set error', () => {
        const state: VorgangState = reducer(initialState, action);

        expect(state.vorgangList.error).toStrictEqual(apiError);
      });
    });
  });

  describe('searchForPreview', () => {
    describe('on "searchForPreview" action', () => {
      const searchString: string = 'searchThisForMe';
      const action = VorgangActions.searchForPreview({ string: searchString });

      it('should set loading to true', () => {
        const state: VorgangState = reducer(initialState, action);

        expect(state.searchPreviewList.loading).toBeTruthy();
      });
    });

    describe('on "searchForPreviewSuccess" action', () => {
      const vorgangList: VorgangListResource = createVorgangListResource();
      const action = VorgangActions.searchForPreviewSuccess({ vorgangList });

      it('should set loading to true', () => {
        const state: VorgangState = reducer(initialState, action);

        expect(state.searchPreviewList).toStrictEqual(createStateResource(vorgangList));
      });
    });

    describe('on "searchForPreviewFailure" action', () => {
      const apiError: ApiError = createApiError();
      const httpErrorResponse: HttpErrorResponse = createHttpErrorResponse(apiError);
      const action = VorgangActions.searchForPreviewFailure({ httpErrorResponse });

      it('should set error', () => {
        const state: VorgangState = reducer(initialState, action);

        expect(state.searchPreviewList.error).toStrictEqual(apiError);
      });
    });
  });

  describe('clearSearchPreviewList', () => {
    describe('on "clearSearchPreviewList" action', () => {
      it('should empty searchPreviewList', () => {
        const action = VorgangActions.clearSearchPreviewList();

        const state: VorgangState = reducer(initialState, action);

        expect(state.searchPreviewList).toStrictEqual(createEmptyStateResource());
      });
    });
  });

  describe('on "updateCurrentRouteData" action', () => {
    it('should call updateByRouteData', () => {
      const spy = jest.spyOn(Reducer, 'updateByRouteData');
      const action = NavigationActions.updateCurrentRouteData({ routeData: createRouteData() });

      Reducer.reducer(initialState, action);

      expect(spy).toHaveBeenCalled();
    });
  });

  describe('updateLocalStorage', () => {
    it('should update filter', () => {
      const spy = jest.spyOn(Storage, 'setFilterIntoStorage');

      Reducer.updateLocalStorage(VorgangFilter.MEINE_VORGAENGE, VorgangView.NEU);

      expect(spy).toHaveBeenCalledWith('meine');
    });

    describe('update view', () => {
      it('should clear view if view is vorgangList', () => {
        const spy = jest.spyOn(Storage, 'removeLocalStorageView');

        Reducer.updateLocalStorage(VorgangFilter.MEINE_VORGAENGE, VorgangView.VORGANG_LIST);

        expect(spy).toHaveBeenCalled();
      });

      it('should set to given view', () => {
        const spy = jest.spyOn(Storage, 'setViewIntoStorage');

        Reducer.updateLocalStorage(VorgangFilter.MEINE_VORGAENGE, VorgangView.ANGENOMMEN);

        expect(spy).toHaveBeenCalledWith('angenommen');
      });
    });
  });

  describe('on "clearSearchString" action', () => {
    it('should empty searchString', () => {
      const action = VorgangActions.clearSearchString();
      const emptySearchString: string = EMPTY_STRING;

      const state: VorgangState = reducer(initialState, action);

      expect(state.searchString).toStrictEqual(emptySearchString);
    });
  });

  //VorgangWithEingang
  describe('vorgangWithEingang', () => {
    describe('on "loadVorgangWithEingang" action', () => {
      const resourceUri: ResourceUri = faker.internet.url();

      it('should set loading to true', () => {
        const action = VorgangActions.loadVorgangWithEingang({ resourceUri });

        const state: VorgangState = reducer(initialState, action);

        expect(state.vorgangWithEingang.loading).toBeTruthy();
      });
    });

    describe('on "loadVorgangWithEingangSuccess" action', () => {
      it('should set vorgangWithEingang', () => {
        const vorgangWithEingang: VorgangWithEingangResource = createVorgangWithEingangResource();
        const action = VorgangActions.loadVorgangWithEingangSuccess({ vorgangWithEingang });

        const state: VorgangState = reducer(initialState, action);

        expect(state.vorgangWithEingang.resource).toStrictEqual(vorgangWithEingang);
      });

      it('should set reload representationList', () => {
        const vorgangWithEingang: VorgangWithEingangResource = createVorgangWithEingangResource();
        const action = VorgangActions.loadVorgangWithEingangSuccess({ vorgangWithEingang });

        const state: VorgangState = reducer(initialState, action);

        expect(state.representationList.reload).toBeTruthy();
      });

      it('should set reload attachmentList', () => {
        const vorgangWithEingang: VorgangWithEingangResource = createVorgangWithEingangResource();
        const action = VorgangActions.loadVorgangWithEingangSuccess({ vorgangWithEingang });

        const state: VorgangState = reducer(initialState, action);

        expect(state.attachmentList.reload).toBeTruthy();
      });
    });

    describe('on "setReloadVorgangWithEingang" action', () => {
      it('should set vorgangWithEingang#reload to true', () => {
        const action = VorgangActions.setReloadVorgangWithEingang();

        const state: VorgangState = reducer(initialState, action);

        expect(state.vorgangWithEingang.reload).toBeTruthy();
      });
    });
  });

  describe('attachmentList', () => {
    describe('on "loadAttachmentList" action', () => {
      const vorgangWithEingang: VorgangWithEingangResource = createVorgangWithEingangResource();

      it('should set state resource loading', () => {
        const action = VorgangActions.loadAttachmentList({ vorgangWithEingang });

        const state: VorgangState = reducer(initialState, action);

        expect(state.attachmentList.loading).toBeTruthy();
      });
    });

    describe('on "loadAttachmentListSuccess" action', () => {
      it('should set state resource', () => {
        const binaryFileList: BinaryFileListResource = createBinaryFileListResource();
        const action = VorgangActions.loadAttachmentListSuccess({ binaryFileList });

        const state: VorgangState = reducer(initialState, action);

        expect(state.attachmentList.resource).toBe(binaryFileList);
      });
    });
  });

  describe('representationList', () => {
    describe('on "loadRepresentationList" action', () => {
      const vorgangWithEingang: VorgangWithEingangResource = createVorgangWithEingangResource();

      it('should set state resource loading', () => {
        const action = VorgangActions.loadRepresentationList({ vorgangWithEingang });

        const state: VorgangState = reducer(initialState, action);

        expect(state.representationList.loading).toBeTruthy();
      });
    });

    describe('on "loadRepresentationListSuccess" action', () => {
      it('should set state resource', () => {
        const binaryFileList: BinaryFileListResource = createBinaryFileListResource();
        const action = VorgangActions.loadRepresentationListSuccess({ binaryFileList });

        const state: VorgangState = reducer(initialState, action);

        expect(state.representationList.resource).toBe(binaryFileList);
      });
    });
  });

  describe('pending commands', () => {
    describe('on "loadPendingCommandListSuccess" action', () => {
      it('should set forward command', () => {
        const forwardCommand: CommandResource = {
          ...createCommandResource(),
          order: CommandOrder.REDIRECT_VORGANG,
        };
        const commandList: CommandListResource = createCommandListResource([
          createCommandResource(),
          forwardCommand,
        ]);
        const action = VorgangActions.loadPendingCommandListSuccess({ commandList });

        const state: VorgangState = reducer(initialState, action);

        expect(state.forwardPendingCommand.resource).toBe(forwardCommand);
      });

      it('should set send postfach nachricht command', () => {
        const sendPostfachNachrichtCommand: CommandResource = {
          ...createCommandResource(),
          order: CommandOrder.SEND_POSTFACH_NACHRICHT,
        };
        const commandList: CommandListResource = createCommandListResource([
          createCommandResource(),
          sendPostfachNachrichtCommand,
        ]);
        const action = VorgangActions.loadPendingCommandListSuccess({ commandList });

        const state: VorgangState = reducer(initialState, action);

        expect(state.sendPostfachNachrichtPendingCommand.resource).toBe(
          sendPostfachNachrichtCommand,
        );
      });
    });
  });

  describe('forwardPendingCommand', () => {
    describe('on "setForwardingSingleCommandLoading" action', () => {
      it('should set forward pending command loading', () => {
        const action = VorgangActions.setForwardingSingleCommandLoading();

        const state: VorgangState = reducer(initialState, action);

        expect(state.forwardPendingCommand.loading).toBeTruthy();
      });
    });

    describe('on "setForwardingSingleCommand" action', () => {
      it('should set forward pending command', () => {
        const commandStateResource: StateResource<CommandResource> =
          createStateResource(createCommandResource());
        const action = VorgangActions.setForwardingSingleCommand({ commandStateResource });

        const state: VorgangState = reducer(initialState, action);

        expect(state.forwardPendingCommand).toBe(commandStateResource);
      });
    });
  });

  describe('sendPostfachNachrichtPendingCommand', () => {
    describe('on "setSendPostfachNachrichtSingleCommandLoading" action', () => {
      it('should set send postfach nachricht pending command loading', () => {
        const action = VorgangActions.setSendPostfachNachrichtSingleCommandLoading();

        const state: VorgangState = reducer(initialState, action);

        expect(state.sendPostfachNachrichtPendingCommand.loading).toBeTruthy();
      });
    });

    describe('on "setSendPostfachNachrichtSingleCommand" action', () => {
      it('should set forward pending command', () => {
        const commandStateResource: StateResource<CommandResource> =
          createStateResource(createCommandResource());
        const action = VorgangActions.setSendPostfachNachrichtSingleCommand({
          commandStateResource,
        });

        const state: VorgangState = reducer(initialState, action);

        expect(state.sendPostfachNachrichtPendingCommand).toBe(commandStateResource);
      });
    });
  });

  describe('initAssignUser', () => {
    it('should set reload true', () => {
      const action = VorgangActions.initAssignUser();

      const state: VorgangState = reducer(initialState, action);

      expect(state.assignUserCommand.reload).toBeTruthy();
    });
  });

  describe('publishConcurrentModificationAction', () => {
    it('should set vorgangWithEingang reload true', () => {
      const action = CommandActions.publishConcurrentModificationAction();

      const state: VorgangState = reducer(initialState, action);

      expect(state.vorgangWithEingang.reload).toBeTruthy();
    });
  });

  describe('createCommand', () => {
    const resource: Resource = createDummyResource();
    const linkRel: ResourceUri = faker.internet.url();
    const command: CreateCommand = createCommand();

    it('should call getStatusCommandMapByCreatedCommand', () => {
      const spy = jest.spyOn(Reducer, 'getStatusCommandMapByCreateCommand');

      const action = CommandActions.createCommand({ resource, linkRel, command });

      reducer(initialState, action);

      expect(spy).toHaveBeenCalled();
    });

    it('should call getAssignUserCommandByCreateCommand', () => {
      const spy = jest.spyOn(Reducer, 'getAssignUserCommandByCreateCommand');

      const action = CommandActions.createCommand({ resource, linkRel, command });

      reducer(initialState, action);

      expect(spy).toHaveBeenCalled();
    });
  });

  describe('createCommandSuccess', () => {
    const command: CommandResource = createCommandResource();

    it('should call getStatusCommandMapByCreateCommandSuccess', () => {
      const spy = jest.spyOn(Reducer, 'getStatusCommandMapByCreateCommandSuccess');

      const action = CommandActions.createCommandSuccess({ command });

      reducer(initialState, action);

      expect(spy).toHaveBeenCalled();
    });

    it('should call getAssignUserCommandByCreateCommandSuccess', () => {
      const spy = jest.spyOn(Reducer, 'getAssignUserCommandByCreateCommandSuccess');

      const action = CommandActions.createCommandSuccess({ command });

      reducer(initialState, action);

      expect(spy).toHaveBeenCalled();
    });

    it('should set vorgangWithEingang reload true', () => {
      const action = CommandActions.createCommandSuccess({ command });

      const state: VorgangState = reducer(initialState, action);

      expect(state.vorgangWithEingang.reload).toBeTruthy();
    });
  });

  describe('revokeCommand', () => {
    const command: CommandResource = createCommandResource();

    it('should set revokeCommand loading true', () => {
      const action = CommandActions.revokeCommand({ command });

      const state: VorgangState = reducer(initialState, action);

      expect(state.revokeCommand.loading).toBeTruthy();
    });
  });

  describe('revokeCommandSuccess', () => {
    const command: CommandResource = createCommandResource();

    it('should clear statusCommandMap', () => {
      const action = CommandActions.revokeCommandSuccess({ command });

      const state: VorgangState = reducer(initialState, action);

      expect(state.statusCommandMap).toEqual({});
    });

    it('should set vorgangWithEingang reload true', () => {
      const action = CommandActions.revokeCommandSuccess({ command });

      const state: VorgangState = reducer(initialState, action);

      expect(state.vorgangWithEingang.reload).toBeTruthy();
    });

    it('should set revoke command', () => {
      const action = CommandActions.revokeCommandSuccess({ command });

      const state: VorgangState = reducer(initialState, action);

      expect(state.revokeCommand.resource).toBe(command);
    });
  });

  describe('getStatusCommandMapByCreateCommand', () => {
    describe('on status command', () => {
      it('should set status order command loading true', () => {
        const command: CommandResource = {
          ...createCommandResource(),
          order: CommandOrder.VORGANG_ANNEHMEN,
        };

        const statusCommandMap: StatusCommandMap = Reducer.getStatusCommandMapByCreateCommand(
          initialState,
          command,
        );

        expect(statusCommandMap[command.order].loading).toBeTruthy();
      });
    });

    describe('on other command', () => {
      it('should return state value', () => {
        const command: CommandResource = { ...createCommandResource(), order: 'quatsch' };

        const statusCommandMap: StatusCommandMap = Reducer.getStatusCommandMapByCreateCommand(
          initialState,
          command,
        );

        expect(statusCommandMap).toBe(initialState.statusCommandMap);
      });
    });
  });

  describe('getStatusCommandMapByCreateCommandSuccess', () => {
    describe('on status command', () => {
      it('should set status order command', () => {
        const command: CommandResource = {
          ...createCommandResource(),
          order: CommandOrder.VORGANG_ANNEHMEN,
        };

        const statusCommandMap: StatusCommandMap =
          Reducer.getStatusCommandMapByCreateCommandSuccess(initialState, command);

        expect(statusCommandMap[command.order].resource).toBe(command);
      });
    });

    describe('on other command', () => {
      it('should return state value', () => {
        const command: CommandResource = { ...createCommandResource(), order: 'quatsch' };

        const statusCommandMap: StatusCommandMap =
          Reducer.getStatusCommandMapByCreateCommandSuccess(initialState, command);

        expect(statusCommandMap).toBe(initialState.statusCommandMap);
      });
    });
  });

  describe('getAssignUserCommandByCreateCommand', () => {
    describe('on assignUser command', () => {
      it('should set command loading true', () => {
        const command: CommandResource = {
          ...createCommandResource(),
          order: CommandOrder.ASSIGN_USER,
        };

        const assignUserCommand: StateResource<CommandResource> =
          Reducer.getAssignUserCommandByCreateCommand(initialState, command);

        expect(assignUserCommand.loading).toBeTruthy();
      });
    });

    describe('on other command', () => {
      it('should return state value', () => {
        const command: CommandResource = { ...createCommandResource(), order: 'quatsch' };

        const assignUserCommand: StateResource<CommandResource> =
          Reducer.getAssignUserCommandByCreateCommand(initialState, command);

        expect(assignUserCommand).toBe(initialState.assignUserCommand);
      });
    });
  });

  describe('getAssignUserCommandByCreateCommandSuccess', () => {
    describe('on assignUser command', () => {
      it('should set command', () => {
        const command: CommandResource = {
          ...createCommandResource(),
          order: CommandOrder.ASSIGN_USER,
        };

        const assignUserCommand: StateResource<CommandResource> =
          Reducer.getAssignUserCommandByCreateCommandSuccess(initialState, command);

        expect(assignUserCommand.resource).toBe(command);
      });
    });

    describe('on other command', () => {
      it('should return state value', () => {
        const command: CommandResource = { ...createCommandResource(), order: 'quatsch' };

        const assignUserCommand: StateResource<CommandResource> =
          Reducer.getAssignUserCommandByCreateCommandSuccess(initialState, command);

        expect(assignUserCommand).toBe(initialState.assignUserCommand);
      });
    });
  });

  describe('vorgangExport', () => {
    const vorgangWithEingang: VorgangWithEingangResource = createVorgangWithEingangResource();

    describe('on exportVorgang action', () => {
      it('should set stateResource loading', () => {
        const action = VorgangActions.exportVorgang({ vorgangWithEingang });

        const state: VorgangState = reducer(initialState, action);

        expect(state.vorgangExport.loading).toBeTruthy();
      });
    });

    describe('on exportVorgangSuccess action', () => {
      it('should clear stateResource', () => {
        const action = VorgangActions.exportVorgangSuccess();

        const state: VorgangState = reducer(
          { ...initialState, vorgangExport: createEmptyStateResource(true) },
          action,
        );

        expect(state.vorgangExport).toEqual(createStateResource(true));
      });
    });
  });

  describe('updateByRouteData', () => {
    describe('contains search route', () => {
      it('should set previewList reload on changed searchString', () => {
        const action = NavigationActions.updateCurrentRouteData({
          routeData: buildCurrentRouteData('search like me'),
        });
        const state: VorgangState = reducer(
          { ...initialState, searchString: 'existingSearchString' },
          action,
        );

        expect(state.searchPreviewList.reload).toBeTruthy();
      });

      it('should keep searchString on same searchString', () => {
        const searchString: string = 'existingSearchString';
        const action = NavigationActions.updateCurrentRouteData({
          routeData: buildCurrentRouteData(searchString),
        });

        const state: VorgangState = reducer({ ...initialState, searchString }, action);

        expect(state.searchString).toEqual(searchString);
      });

      function buildCurrentRouteData(searchString: string): RouteData {
        const urlSegments: UrlSegment[] = [
          <any>{ path: MEINE_ROUTE_PARAM },
          <any>{ path: ROUTE_PARAM_BY_VORGANG_VIEW[VorgangView.SEARCH] },
          <any>{ path: searchString },
        ];
        const queryParameter = { ['search']: searchString };
        return { ...createRouteData(), urlSegments, queryParameter };
      }
    });

    describe('contains vorgangList route', () => {
      describe('localStorage', () => {
        it('should be updated by given vorgangFilter and vorgangView', () => {
          const urlSegments: UrlSegment[] = [
            <any>{ path: ROUTE_PARAM_BY_VORGANG_FILTER[VorgangFilter.ALLE] },
            { path: ROUTE_PARAM_BY_VORGANG_VIEW[VorgangView.ANGENOMMEN] },
          ];
          const routeData: RouteData = { ...createRouteData(), urlSegments };
          const localStorageSpy = jest.spyOn(Reducer, 'updateLocalStorage');

          Reducer.updateByRouteData(initialState, routeData);

          expect(localStorageSpy).toHaveBeenCalledWith(VorgangFilter.ALLE, VorgangView.ANGENOMMEN);
        });

        it('should be updated by given vorgangFilter and default view', () => {
          const urlSegments: UrlSegment[] = [
            <any>{ path: ROUTE_PARAM_BY_VORGANG_FILTER[VorgangFilter.ALLE] },
          ];
          const routeData: RouteData = { ...createRouteData(), urlSegments };
          const localStorageSpy = jest.spyOn(Reducer, 'updateLocalStorage');

          Reducer.updateByRouteData(initialState, routeData);

          expect(localStorageSpy).toHaveBeenCalledWith(
            VorgangFilter.ALLE,
            VorgangView.VORGANG_LIST,
          );
        });
      });

      describe('filter only', () => {
        const urlSegments: UrlSegment[] = [
          <any>{ path: ROUTE_PARAM_BY_VORGANG_FILTER[VorgangFilter.ALLE] },
        ];
        const routeData: RouteData = { ...createRouteData(), urlSegments };

        it('should set reload vorgangList', () => {
          const state: VorgangState = Reducer.updateByRouteData(initialState, routeData);

          expect(state.vorgangList.reload).toBeTruthy();
        });

        it('should set vorgangView', () => {
          const state: VorgangState = Reducer.updateByRouteData(initialState, routeData);

          expect(state.vorgangView).toBe(VorgangView.VORGANG_LIST);
        });

        it('should set vorgangFilter', () => {
          const routeData: RouteData = {
            ...createRouteData(),
            urlSegments: [
              <any>{ path: ROUTE_PARAM_BY_VORGANG_FILTER[VorgangFilter.MEINE_VORGAENGE] },
            ],
          };

          const state: VorgangState = Reducer.updateByRouteData(initialState, routeData);

          expect(state.vorgangFilter).toBe(VorgangFilter.MEINE_VORGAENGE);
        });

        it('should clear vorgaenge', () => {
          const state: VorgangState = Reducer.updateByRouteData(initialState, routeData);

          expect(state.vorgaenge).toBe(EMPTY_ARRAY);
        });
      });

      describe('filter and view', () => {
        const urlSegments: UrlSegment[] = [
          <any>{ path: ROUTE_PARAM_BY_VORGANG_FILTER[VorgangFilter.MEINE_VORGAENGE] },
          <any>{ path: ROUTE_PARAM_BY_VORGANG_VIEW[VorgangView.NEU] },
        ];
        const routeData: RouteData = { ...createRouteData(), urlSegments };

        it('should set reload vorgangList', () => {
          const state: VorgangState = Reducer.updateByRouteData(initialState, routeData);

          expect(state.vorgangList.reload).toBeTruthy();
        });

        it('should set vorgangView', () => {
          const state: VorgangState = Reducer.updateByRouteData(initialState, routeData);

          expect(state.vorgangView).toBe(VorgangView.NEU);
        });

        it('should set VorgangFilter', () => {
          const state: VorgangState = Reducer.updateByRouteData(initialState, routeData);

          expect(state.vorgangFilter).toBe(VorgangFilter.MEINE_VORGAENGE);
        });

        it('should clear vorgaenge', () => {
          const state: VorgangState = Reducer.updateByRouteData(initialState, routeData);

          expect(state.vorgaenge).toBe(EMPTY_ARRAY);
        });
      });
    });

    describe('contains vorgang route', () => {
      const queryParameter = { [VORGANG_WITH_EINGANG_ROUTE_PARAM]: faker.internet.url() };
      const routeData: RouteData = { ...createRouteData(), queryParameter };

      it('should set reload to vorgangWithEingang', () => {
        const state: VorgangState = Reducer.updateByRouteData(initialState, routeData);

        expect(state.vorgangWithEingang).toEqual(createEmptyStateResource());
      });

      it('should clear forwardPendingCommand', () => {
        const state: VorgangState = Reducer.updateByRouteData(initialState, routeData);

        expect(state.forwardPendingCommand).toEqual(createEmptyStateResource());
      });

      it('should clear sendPostfachNachrichtPendingCommand', () => {
        const state: VorgangState = Reducer.updateByRouteData(initialState, routeData);

        expect(state.sendPostfachNachrichtPendingCommand).toEqual(createEmptyStateResource());
      });
    });
  });

  describe('initialState', () => {
    describe('vorgangStatistic', () => {
      describe('byStatus', () => {
        it('should have null as neu', () => {
          const vorgangStatistic: StateResource<VorgangStatistic> =
            Reducer.initialState.vorgangStatistic;

          expect(vorgangStatistic.resource.byStatus.neu).toBeNull();
        });

        it('should have null as angenommen', () => {
          const vorgangStatistic: StateResource<VorgangStatistic> =
            Reducer.initialState.vorgangStatistic;

          expect(vorgangStatistic.resource.byStatus.angenommen).toBeNull();
        });

        it('should have null as inBearbeitung', () => {
          const vorgangStatistic: StateResource<VorgangStatistic> =
            Reducer.initialState.vorgangStatistic;

          expect(vorgangStatistic.resource.byStatus.inBearbeitung).toBeNull();
        });

        it('should have null as beschieden', () => {
          const vorgangStatistic: StateResource<VorgangStatistic> =
            Reducer.initialState.vorgangStatistic;

          expect(vorgangStatistic.resource.byStatus.beschieden).toBeNull();
        });

        it('should have null as abgeschlossen', () => {
          const vorgangStatistic: StateResource<VorgangStatistic> =
            Reducer.initialState.vorgangStatistic;

          expect(vorgangStatistic.resource.byStatus.abgeschlossen).toBeNull();
        });

        it('should have null as verworfen', () => {
          const vorgangStatistic: StateResource<VorgangStatistic> =
            Reducer.initialState.vorgangStatistic;

          expect(vorgangStatistic.resource.byStatus.verworfen).toBeNull();
        });

        it('should have null as zuLoeschen', () => {
          const vorgangStatistic: StateResource<VorgangStatistic> =
            Reducer.initialState.vorgangStatistic;

          expect(vorgangStatistic.resource.byStatus.zuLoeschen).toBeNull();
        });
      });

      it('should have null as wiedervorlagen', () => {
        const vorgangStatistic: StateResource<VorgangStatistic> =
          Reducer.initialState.vorgangStatistic;

        expect(vorgangStatistic.resource.wiedervorlagen).toBeNull();
      });

      it('should have false as existsWiedervorlageOverdue', () => {
        const vorgangStatistic: StateResource<VorgangStatistic> =
          Reducer.initialState.vorgangStatistic;

        expect(vorgangStatistic.resource.existsWiedervorlageOverdue).toBeFalsy();
      });
    });
  });
});
