package de.ozgcloud.alfa.historie;

import static de.ozgcloud.alfa.historie.AssignedUserChangeHistoryBuilder.*;
import static de.ozgcloud.alfa.historie.ChangeHistoryBuilderTest.*;
import static org.assertj.core.api.Assertions.*;
import static org.mockito.Mockito.*;

import java.time.LocalDateTime;
import java.util.List;
import java.util.UUID;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Spy;

import com.thedeanda.lorem.LoremIpsum;

import de.ozgcloud.alfa.common.command.Command;
import de.ozgcloud.alfa.common.command.CommandOrder;
import de.ozgcloud.alfa.common.command.CommandTestFactory;
import de.ozgcloud.alfa.common.user.UserId;
import de.ozgcloud.alfa.common.user.UserProfile;
import de.ozgcloud.alfa.common.user.UserProfileTestFactory;

public class AssignedUserChangeHistoryBuilderTest {

	private static final String ORGANISATIONSEINHEITEN_ID = UUID.randomUUID().toString();

	private final Command previousCommand = commandFinishedAt(LocalDateTime.of(2023, 5, 1, 12, 0));
	private final Command command = commandFinishedAt(LocalDateTime.of(2023, 6, 1, 12, 0));
	private final List<Command> commands = List.of(previousCommand, command);

	@Mock
	private UserProfileCache userProfileCache;

	@Spy
	@InjectMocks
	private AssignedUserChangeHistoryBuilder builder = AssignedUserChangeHistoryBuilder.builder()
			.withCommands(commands)
			.withOrganisationseinheitenID(ORGANISATIONSEINHEITEN_ID);

	@Nested
	class TestIsRelevant {

		@Test
		void shouldReturnTrue() {
			var command = CommandTestFactory.createBuilder()
					.order(CommandOrder.ASSIGN_USER)
					.build();

			var result = builder.isRelevant(command);

			assertThat(result).isTrue();
		}

		@Test
		void shouldReturnFalse() {
			var command = CommandTestFactory.createBuilder()
					.order(CommandOrder.VORGANG_WIEDEREROEFFNEN)
					.build();

			var result = builder.isRelevant(command);

			assertThat(result).isFalse();
		}
	}

	@Nested
	class TestToCommandWithChangeValues {

		private static final String ASSIGNED_USER_BEFORE_CHANGE = LoremIpsum.getInstance().getWords(2);
		private static final String ASSIGNED_USER_AFTER_CHANGE = LoremIpsum.getInstance().getWords(2);

		private final CommandWithPrevious commandWithPrevious = CommandWithPreviousTestFactory.createBuilder().command(command).previous(previousCommand).build();

		@BeforeEach
		void init() {
			doReturn(ASSIGNED_USER_BEFORE_CHANGE).when(builder).getAssignedUserBeforeChange(commandWithPrevious);
			doReturn(ASSIGNED_USER_AFTER_CHANGE).when(builder).getAssignedUserAfterChange(commandWithPrevious);
		}

		@Test
		void shouldSetCommand() {
			var commandWithChangeValues = callBuilder();

			assertThat(commandWithChangeValues.command()).isEqualTo(command);
		}

		@Test
		void shouldSetValueBeforeChange() {
			var commandWithChangeValues = callBuilder();

			assertThat(commandWithChangeValues.valueBeforeChange()).isEqualTo(ASSIGNED_USER_BEFORE_CHANGE);
		}

		@Test
		void shouldSetValueAfterChange() {
			var commandWithChangeValues = callBuilder();

			assertThat(commandWithChangeValues.valueAfterChange()).isEqualTo(ASSIGNED_USER_AFTER_CHANGE);
		}

		private ChangeHistoryBuilder.CommandWithChangeValues callBuilder() {
			return builder.toCommandWithChangeValues(commandWithPrevious);
		}
	}

	@Nested
	class TestGetAssignedUserBeforeChange {

		private static final String EXPECTED_ASSIGNED_USER = LoremIpsum.getInstance().getWords(2);

		private final CommandWithPrevious commandWithoutPrevious = CommandWithPreviousTestFactory.createBuilder().command(previousCommand).previous(null).build();
		private final CommandWithPrevious commandWithPrevious = CommandWithPreviousTestFactory.createBuilder().command(command).previous(previousCommand).build();

		@Test
		void shouldReturnEmptyStringIfPreviousIsNull() {
			var assignedUser = builder.getAssignedUserBeforeChange(commandWithoutPrevious);

			assertThat(assignedUser).isEmpty();
		}

		@Test
		void shouldGetAssignedUserFromPreviousCommand() {
			givenAssignedUserFromPreviousCommand();

			callBuilder();

			verify(builder).getAssignedUserFullNameFromCommand(previousCommand);
		}

		@Test
		void shouldReturnAssignedUserRetrievedFromPreviousCommand() {
			givenAssignedUserFromPreviousCommand();

			var assignedUser = callBuilder();

			assertThat(assignedUser).isEqualTo(EXPECTED_ASSIGNED_USER);
		}

		private void givenAssignedUserFromPreviousCommand() {
			doReturn(EXPECTED_ASSIGNED_USER).when(builder).getAssignedUserFullNameFromCommand(previousCommand);
		}

		private String callBuilder() {
			return builder.getAssignedUserBeforeChange(commandWithPrevious);
		}
	}

	@Nested
	class TestGetAssignedUserAfterChange {

		private static final String EXPECTED_ASSIGNED_USER = LoremIpsum.getInstance().getWords(2);

		private final CommandWithPrevious commandWithPrevious = CommandWithPreviousTestFactory.createBuilder().command(command).previous(previousCommand).build();

		@Test
		void shouldGetAssignedUserFromCommand() {
			givenAssignedUserFromCurrentCommand();

			callBuilder();

			verify(builder).getAssignedUserFullNameFromCommand(command);
		}

		@Test
		void shouldReturnAssignedUserRetrievedFromCommand() {
			givenAssignedUserFromCurrentCommand();

			var assignedUser = callBuilder();

			assertThat(assignedUser).isEqualTo(EXPECTED_ASSIGNED_USER);
		}

		private void givenAssignedUserFromCurrentCommand() {
			doReturn(EXPECTED_ASSIGNED_USER).when(builder).getAssignedUserFullNameFromCommand(command);
		}

		private String callBuilder() {
			return builder.getAssignedUserAfterChange(commandWithPrevious);
		}
	}

	@Nested
	class TestGetAssignedUserFullNameFromCommand {

		private static final UserId USER_ID = UserId.from(UUID.randomUUID());
		private static final UserProfile USER_PROFILE = UserProfileTestFactory.create();
		private final Command command = previousCommand;

		@BeforeEach
		void init() {
			when(builder.getValueFromCommandBody(BODY_PROPERTY_ASSIGNED_USER, previousCommand)).thenReturn(USER_ID.toString());
			when(userProfileCache.getUserProfile(USER_ID)).thenReturn(USER_PROFILE);
		}

		@Test
		void shouldGetUserIdFromCommand() {
			callBuilder();

			verify(builder).getValueFromCommandBody(BODY_PROPERTY_ASSIGNED_USER, command);
		}

		@Test
		void shouldGetUserProfileForUserId() {
			callBuilder();

			verify(userProfileCache).getUserProfile(USER_ID);
		}

		@Test
		void shouldReturnUserFullName() {
			var assignedUser = callBuilder();

			assertThat(assignedUser).isEqualTo(UserProfileTestFactory.FULLNAME);
		}

		private String callBuilder() {
			return builder.getAssignedUserFullNameFromCommand(command);
		}
	}
}

