package de.ozgcloud.alfa.export;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.nio.charset.StandardCharsets;
import java.util.List;
import java.util.Optional;
import java.util.stream.Stream;
import java.util.zip.ZipEntry;
import java.util.zip.ZipOutputStream;

import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Service;

import de.ozgcloud.alfa.common.ExportFilenameGenerator;
import de.ozgcloud.alfa.common.file.OzgFile;
import de.ozgcloud.alfa.file.ExportFileService;
import de.ozgcloud.alfa.historie.ExportHistorieService;
import de.ozgcloud.alfa.kommentar.ExportKommentarService;
import de.ozgcloud.alfa.vorgang.Eingang;
import de.ozgcloud.alfa.vorgang.EingangHeader;
import de.ozgcloud.alfa.vorgang.ExportVorgangService;
import de.ozgcloud.alfa.vorgang.VorgangWithEingang;
import de.ozgcloud.common.binaryfile.TempFileUtils;
import de.ozgcloud.common.errorhandling.TechnicalException;
import de.xoev.xdomea.AbgabeAbgabe0401;
import lombok.RequiredArgsConstructor;

@RequiredArgsConstructor
@Service
class ExportService {

	static final String EXPORT_FILENAME_SUFFIX = "_Abgabe.Abgabe.0401.xml";
	private static final String EXPORT_FILENAME_TEMPLATE = "%s" + EXPORT_FILENAME_SUFFIX;

	private final XdomeaXmlMarshaller xDomeaXmlMarshaller;

	private final ExportFileService exportFileService;
	private final ExportVorgangService exportVorgangService;
	private final ExportHistorieService exportHistorieService;
	private final ExportKommentarService exportKommentarService;

	public void writeExport(String vorgangId, String filenameId, OutputStream out) {
		var exportData = collectExportData(vorgangId, filenameId);
		var zipFile = createZipFile(exportData);
		writeZipFileContent(zipFile, out);
	}

	ExportData collectExportData(String vorgangId, String filenameId) {
		var vorgang = exportVorgangService.getVorgang(vorgangId);
		var representations = exportFileService.getRepresentations(vorgang).toList();
		var attachments = exportFileService.getAttachments(vorgang).toList();
		var formEngineName = getFormEngineName(vorgang);
		var abgabe = XdomeaNachrichtBuilder.builder()
				.withKopf(exportVorgangService.createKopf(vorgang))
				.withVorgang(exportVorgangService.createVorgangType(vorgang))
				.withAktenzeichen(exportVorgangService.createAkteType(vorgang))
				.withRepresentations(exportFileService.createDokumentTypes(representations, formEngineName).toList())
				.withAttachments(exportFileService.createDokumentTypes(attachments, formEngineName).toList())
				.withHistorie(exportHistorieService.createHistorienProtokollInformationTypes(vorgang).toList())
				.withKommentare(exportKommentarService.createDokumentTypes(vorgang).toList())
				.build();
		var exportFiles = Stream.concat(representations.stream(), attachments.stream()).toList();
		return ExportData.builder().abgabe(abgabe).exportFilename(buildXmlFilename(filenameId)).exportFiles(exportFiles).build();
	}

	String getFormEngineName(VorgangWithEingang vorgang) {
		return Optional.ofNullable(vorgang.getEingang())
				.map(Eingang::getHeader)
				.map(EingangHeader::getFormEngineName).orElse(StringUtils.EMPTY);
	}

	String buildXmlFilename(String filenameId) {
		return String.format(EXPORT_FILENAME_TEMPLATE, filenameId);
	}

	String createXmlContent(AbgabeAbgabe0401 abgabe) {
		return xDomeaXmlMarshaller.marshal(abgabe);
	}

	File createZipFile(ExportData exportData) {
		var file = TempFileUtils.createTmpFile().toFile();
		try (var zipOutputStream = new ZipOutputStream(new FileOutputStream(file))) {
			putZipEntry(exportData.getExportFilename(), createXmlContent(exportData.getAbgabe()), zipOutputStream);
			putFilesIntoZip(exportData.getExportFiles(), zipOutputStream);
			return file;
		} catch (Exception e) {
			throw new TechnicalException("Error creating xdomea zip file", e);
		}
	}

	void putZipEntry(String fileName, String fileData, ZipOutputStream zipOutputStream) throws IOException {
		var entry = new ZipEntry(fileName);
		zipOutputStream.putNextEntry(entry);
		zipOutputStream.write(fileData.getBytes(StandardCharsets.UTF_8));
		zipOutputStream.closeEntry();
	}

	private void putFilesIntoZip(List<OzgFile> ozgFiles, ZipOutputStream zipOutputStream) {
		ozgFiles.forEach(ozgFile -> putOzgFileIntoZip(ozgFile, zipOutputStream));
	}

	void putOzgFileIntoZip(OzgFile ozgFile, ZipOutputStream zipOutputStream) {
		try {
			var entry = new ZipEntry(ExportFilenameGenerator.generateExportFilename(ozgFile));
			zipOutputStream.putNextEntry(entry);
			exportFileService.writeOzgFile(ozgFile.getId(), zipOutputStream);
			zipOutputStream.closeEntry();
		} catch (IOException e) {
			throw new TechnicalException("Cannot add file to ZIP.", e);
		}
	}

	void writeZipFileContent(File file, OutputStream outputStream) {
		try (var fileInputStream = new FileInputStream(file)) {
			fileInputStream.transferTo(outputStream);
		} catch (Exception e) {
			throw new TechnicalException("Error writing xdomea zip file to output stream", e);
		}
	}
}
