package de.ozgcloud.alfa.file;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.Mockito.*;

import org.apache.commons.lang3.RandomUtils;
import org.apache.commons.lang3.StringUtils;
import org.junit.jupiter.api.AfterEach;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.Mock;
import org.mockito.MockedStatic;
import org.mockito.Spy;

import de.ozgcloud.alfa.common.FormatTypeTestFactory;
import de.ozgcloud.alfa.common.PrimaerdokumentTypeBuilder;
import de.ozgcloud.alfa.common.PrimaerdokumentTypeTestFactory;
import de.ozgcloud.alfa.common.TestUtils;
import de.ozgcloud.alfa.common.UUIDConverter;
import de.ozgcloud.alfa.common.VersionTypeTestFactory;
import de.ozgcloud.alfa.common.file.OzgFile;
import de.ozgcloud.alfa.common.file.OzgFileTestFactory;
import de.ozgcloud.alfa.export.AllgemeineMetadatenTypeTestFactory;
import de.ozgcloud.alfa.export.IdentifikationObjektTypeTestFactory;
import de.xoev.xdomea.AllgemeineMetadatenType;
import de.xoev.xdomea.FormatType;
import de.xoev.xdomea.IdentifikationObjektType;
import de.xoev.xdomea.PrimaerdokumentType;
import de.xoev.xdomea.VersionType;

class DokumentTypeBuilderTest {

	private static final Long DOKUMENT_ORDINAL_NUMBER = RandomUtils.nextLong();
	private static final String FORM_ENGINE_NAME = "DUMMY_NAME";
	private final OzgFile ozgFile = OzgFileTestFactory.create();

	@Spy
	private final DokumentTypeBuilder builder = DokumentTypeBuilder.builder()
			.withOzgFile(ozgFile)
			.withFormEngineName(FORM_ENGINE_NAME)
			.withOrdinalNumber(DOKUMENT_ORDINAL_NUMBER);

	private MockedStatic<UUIDConverter> uuidConverter;

	@BeforeEach
	void init() {
		uuidConverter = mockStatic(UUIDConverter.class);
	}

	@AfterEach
	void cleanup() {
		uuidConverter.close();
	}

	@Nested
	class TestBuild {

		private final IdentifikationObjektType identifikationObjekt = IdentifikationObjektTypeTestFactory.create();
		private final AllgemeineMetadatenType allgemeineMetadaten = AllgemeineMetadatenTypeTestFactory.create();
		private final VersionType versionType = VersionTypeTestFactory.create();
		private final PrimaerdokumentType primaerdokument = PrimaerdokumentTypeTestFactory.create();

		@BeforeEach
		void setUp() {
			doReturn(identifikationObjekt).when(builder).createIdentifikation();
			doReturn(versionType).when(builder).createVersionType();
			doReturn(allgemeineMetadaten).when(builder).createAllgemeineMetadaten();
			doReturn(primaerdokument).when(builder).createPrimaerdokument();
		}

		@Test
		void shouldCreateIdentifikation() {
			builder.build();

			verify(builder).createIdentifikation();
		}

		@Test
		void shouldHaveIdentifikation() {
			var dokument = builder.build();

			assertThat(dokument.getIdentifikation()).isEqualTo(identifikationObjekt);
		}

		@Test
		void shouldCreateAllgemeineMetadaten() {
			builder.build();

			verify(builder).createAllgemeineMetadaten();
		}

		@Test
		void shouldHaveAllgemeineMetadaten() {
			var dokument = builder.build();

			assertThat(dokument.getAllgemeineMetadaten()).isEqualTo(allgemeineMetadaten);
		}

		@Test
		void shouldCreateVersion() {
			builder.build();

			verify(builder).createVersionType();
		}

		@Test
		void shouldHaveVersion() {
			var dokumentType = builder.build();

			assertThat(dokumentType.getVersion()).containsExactly(versionType);
		}

		@Test
		void shouldCreatePrimaerdokument() {
			builder.build();

			verify(builder).createPrimaerdokument();
		}

		@Test
		void shouldHavePrimaerdokument() {
			var dokument = builder.build();

			assertThat(dokument.getVersion().get(0).getFormat().get(0).getPrimaerdokument()).isEqualTo(primaerdokument);
		}

	}

	@Nested
	class TestCreateIdentifikation {

		private final String FILE_UUID = "64a820d3-6285-172a-c028-0000000026d0";

		@BeforeEach
		void init() {
			uuidConverter.when(() -> UUIDConverter.fromObjectId(OzgFileTestFactory.ID.toString())).thenReturn(FILE_UUID);
		}

		@Test
		void shouldConvertObjectId() {
			builder.createIdentifikation();

			uuidConverter.verify(() -> UUIDConverter.fromObjectId(OzgFileTestFactory.ID.toString()));
		}

		@Test
		void shouldHaveId() {
			var identifikation = builder.createIdentifikation();

			assertThat(identifikation.getID()).matches(TestUtils.UUID_REGEX).isEqualTo(FILE_UUID);
		}

		@Test
		void shouldHaveNummerImUebergeordnetenContainer() {
			var identifikation = builder.createIdentifikation();

			assertThat(identifikation.getNummerImUebergeordnetenContainer()).isEqualTo(DOKUMENT_ORDINAL_NUMBER);
		}
	}

	@Nested
	class TestCreateAllgemeineMetadaten {

		@Test
		void shouldHaveBetreff() {
			var allgemeineMetadaten = builder.createAllgemeineMetadaten();

			assertThat(allgemeineMetadaten.getBetreff()).isEqualTo(OzgFileTestFactory.NAME);
		}

		@Test
		void shouldHaveEmptyKennzeichen() {
			var allgemeineMetadaten = builder.createAllgemeineMetadaten();

			assertThat(allgemeineMetadaten.getKennzeichen()).isEmpty();
		}

		@Test
		void shouldHaveEmptyBemerkung() {
			var allgemeineMetadaten = builder.createAllgemeineMetadaten();

			assertThat(allgemeineMetadaten.getBemerkung()).isEmpty();
		}

		@Test
		void shouldHaveMedium() {
			var allgemeineMetadaten = builder.createAllgemeineMetadaten();

			assertThat(allgemeineMetadaten.getMedium().getCode()).isEqualTo(DokumentTypeBuilder.ALLGEMEINE_METADATEN_MEDIUM_CODE);
		}

	}

	@Nested
	class TestCreateFormatType {

		@Test
		void shouldHaveName() {
			var formatType = builder.createFormatType();

			assertThat(formatType.getName().getCode()).isEqualTo(DateiformatCode.PDF.getCode());
		}

		@Test
		void shouldHaveSonstinerNameEmptyString() {
			var formatType = builder.createFormatType();

			assertThat(formatType.getSonstigerName()).isEqualTo(StringUtils.EMPTY);
		}

		@Test
		void shouldHaveVersionEmptyString() {
			var formatType = builder.createFormatType();

			assertThat(formatType.getVersion()).isEqualTo(StringUtils.EMPTY);
		}

	}

	@Nested
	class CreateVersionType {

		private final FormatType formatType = FormatTypeTestFactory.create();

		@BeforeEach
		void setUp() {
			doReturn(formatType).when(builder).createFormatType();
		}

		@Test
		void shouldHaveNummer() {
			var versionType = builder.createVersionType();

			assertThat(versionType.getNummer()).isEqualTo(DokumentTypeBuilder.VERSION_NUMMER);
		}

		@Test
		void shouldCreateFormatType() {
			builder.createVersionType();

			verify(builder).createFormatType();
		}

		@Test
		void shouldHaveFormat() {
			var versionType = builder.createVersionType();

			assertThat(versionType.getFormat()).first().isEqualTo(formatType);
		}
	}

	@Nested
	class TestCreatePrimaerdokument {

		private MockedStatic<PrimaerdokumentTypeBuilder> primaerdokumentTypeBuilderMockedStatic;
		@Mock
		private PrimaerdokumentTypeBuilder primaerdokumentTypeBuilder;

		private final PrimaerdokumentType primaerdokumentType = PrimaerdokumentTypeTestFactory.create();

		@BeforeEach
		void setUp() {
			primaerdokumentTypeBuilderMockedStatic = mockStatic(PrimaerdokumentTypeBuilder.class);
			primaerdokumentTypeBuilderMockedStatic.when(PrimaerdokumentTypeBuilder::builder).thenReturn(primaerdokumentTypeBuilder);
			when(primaerdokumentTypeBuilder.withOzgFile(ozgFile)).thenReturn(primaerdokumentTypeBuilder);
			when(primaerdokumentTypeBuilder.withErsteller(FORM_ENGINE_NAME)).thenReturn(primaerdokumentTypeBuilder);
			when(primaerdokumentTypeBuilder.build()).thenReturn(primaerdokumentType);
		}

		@AfterEach
		void tearDown() {
			primaerdokumentTypeBuilderMockedStatic.close();
		}

		@Test
		void shouldCreatePrimaerdokumentWithOzgFile() {
			builder.createPrimaerdokument();

			verify(primaerdokumentTypeBuilder).withOzgFile(ozgFile);
		}

		@Test
		void shouldCreatePrimaerdokumentWithErsteller() {
			builder.createPrimaerdokument();

			verify(primaerdokumentTypeBuilder).withErsteller(FORM_ENGINE_NAME);
		}

		@Test
		void shouldBuildPrimaerdokument() {
			builder.createPrimaerdokument();

			verify(primaerdokumentTypeBuilder).build();
		}

		@Test
		void shouldReturnBuildPrimaerdokument() {
			var primaerdokument = builder.createPrimaerdokument();

			assertThat(primaerdokument).isEqualTo(primaerdokumentType);
		}
	}

}